@@ -1,13 +1,18 @@
 import json
-from json import dumps
-from typing import Any, Optional, List, Dict
+import math
+import types
+from typing import Any, Optional, List, Dict, Callable, Generator
+
+from langchain_core.documents import Document
 from pydantic import BaseModel, model_validator, Field
-from langchain_core.tools import ToolException
 from ..langchain.tools.vector import VectorAdapter
 from langchain_core.messages import HumanMessage
 from alita_sdk.tools.elitea_base import BaseToolApiWrapper
 from logging import getLogger
 
+from ..utils.logging import dispatch_custom_event
+from ..utils.utils import IndexerKeywords
+
 logger = getLogger(__name__)
 
 class IndexDocumentsModel(BaseModel):
@@ -139,6 +144,7 @@ class VectorStoreWrapper(BaseToolApiWrapper):
     vectoradapter: Any = None
     pg_helper: Any = None
     embeddings: Any = None
+    process_document_func: Optional[Callable] = None
     
     @model_validator(mode='before')
     @classmethod
@@ -182,18 +188,212 @@ class VectorStoreWrapper(BaseToolApiWrapper):
             except Exception as e:
                 logger.error(f"Failed to initialize PGVectorSearch: {str(e)}")
 
-    def index_documents(self, documents):
+    def _clean_collection(self):
+        """
+        Clean the vectorstore collection by deleting all indexed data.
+        """
+        self._log_data(
+            f"Cleaning collection '{self.dataset}'",
+            tool_name="_clean_collection"
+        )
+        data = self.vectoradapter.vectorstore.get(include=['metadatas'])
+        if data['ids']:
+            self.vectoradapter.vectorstore.delete(ids=data['ids'])
+        self._log_data(
+            f"Collection '{self.dataset}' has been cleaned. ",
+            tool_name="_clean_collection"
+        )
+
+    def _get_indexed_data(self, store):
+        """ Get all indexed data from vectorstore for non-code content """
+
+        # get already indexed data
+        result = {}
+        try:
+            self._log_data("Retrieving already indexed data from vectorstore",
+                           tool_name="index_documents")
+            data = store.get(include=['metadatas'])
+            # re-structure data to be more usable
+            for meta, db_id in zip(data['metadatas'], data['ids']):
+                # get document id from metadata
+                doc_id = str(meta['id'])
+                dependent_docs = meta.get(IndexerKeywords.DEPENDENT_DOCS.value, [])
+                if dependent_docs:
+                    dependent_docs = [d.strip() for d in dependent_docs.split(';') if d.strip()]
+                parent_id = meta.get(IndexerKeywords.PARENT.value, -1)
+                #
+                chunk_id = meta.get('chunk_id')
+                if doc_id in result and chunk_id:
+                    # if document with the same id already saved, add db_id fof current one as chunk
+                    result[doc_id]['all_chunks'].append(db_id)
+                else:
+                    result[doc_id] = {
+                        'metadata': meta,
+                        'id': db_id,
+                        'all_chunks': [db_id],
+                        IndexerKeywords.DEPENDENT_DOCS.value: dependent_docs,
+                        IndexerKeywords.PARENT.value: parent_id
+                    }
+        except Exception as e:
+            logger.error(f"Failed to get indexed data from vectorstore: {str(e)}. Continuing with empty index.")
+        return result
+
+    def _get_code_indexed_data(self, store) -> Dict[str, Dict[str, Any]]:
+        """ Get all indexed data from vectorstore for code content """
+
+        # get already indexed data
+        result = {}
+        try:
+            self._log_data("Retrieving already indexed code data from vectorstore",
+                           tool_name="index_documents")
+            data = store.get(include=['metadatas'])
+            # re-structure data to be more usable
+            for meta, db_id in zip(data['metadatas'], data['ids']):
+                filename = meta['filename']
+                commit_hash = meta.get('commit_hash')
+                if filename not in result:
+                    result[filename] = {
+                        'commit_hashes': [],
+                        'ids': []
+                    }
+                if commit_hash is not None:
+                    result[filename]['commit_hashes'].append(commit_hash)
+                result[filename]['ids'].append(db_id)
+        except Exception as e:
+            logger.error(f"Failed to get indexed code data from vectorstore: {str(e)}. Continuing with empty index.")
+        return result
+
+    def _reduce_duplicates(
+            self,
+            documents: Generator[Any, None, None],
+            store,
+            get_indexed_data: Callable,
+            key_fn: Callable,
+            compare_fn: Callable,
+            remove_ids_fn: Callable,
+            log_msg: str = "Verification of documents to index started"
+    ) -> List[Any]:
+        """Generic duplicate reduction logic for documents."""
+        self._log_data(log_msg, tool_name="index_documents")
+        indexed_data = get_indexed_data(store)
+        indexed_keys = set(indexed_data.keys())
+        if not indexed_keys:
+            self._log_data("Vectorstore is empty, indexing all incoming documents", tool_name="index_documents")
+            return list(documents)
+
+        final_docs = []
+        docs_to_remove = set()
+
+        for document in documents:
+            key = key_fn(document)
+            if key in indexed_keys:
+                if compare_fn(document, indexed_data[key]):
+                    continue
+                final_docs.append(document)
+                docs_to_remove.update(remove_ids_fn(indexed_data, key))
+            else:
+                final_docs.append(document)
+
+        if docs_to_remove:
+            self._log_data(
+                f"Removing {len(docs_to_remove)} documents from vectorstore that are already indexed with different updated_on.",
+                tool_name="index_documents"
+            )
+            store.delete(ids=list(docs_to_remove))
+
+        return final_docs
+
+    def _reduce_non_code_duplicates(self, documents: Generator[Any, None, None], store) -> List[Any]:
+        return self._reduce_duplicates(
+            documents,
+            store,
+            self._get_indexed_data,
+            lambda doc: doc.metadata.get('id'),
+            lambda doc, idx: (
+                    doc.metadata.get('updated_on') and
+                    idx['metadata'].get('updated_on') and
+                    doc.metadata.get('updated_on') == idx['metadata'].get('updated_on')
+            ),
+            lambda idx_data, key: (
+                    idx_data[key]['all_chunks'] +
+                    [idx_data[dep_id]['id'] for dep_id in idx_data[key][IndexerKeywords.DEPENDENT_DOCS.value]] +
+                    [chunk_db_id for dep_id in idx_data[key][IndexerKeywords.DEPENDENT_DOCS.value]
+                     for chunk_db_id in idx_data[dep_id]['all_chunks']]
+            ),
+            log_msg="Verification of documents to index started"
+        )
+
+    def _reduce_code_duplicates(self, documents: Generator[Any, None, None], store) -> List[Any]:
+        return self._reduce_duplicates(
+            documents,
+            store,
+            self._get_code_indexed_data,
+            lambda doc: doc.metadata.get('filename'),
+            lambda doc, idx: (
+                    doc.metadata.get('commit_hash') and
+                    idx.get('commit_hashes') and
+                    doc.metadata.get('commit_hash') in idx.get('commit_hashes')
+            ),
+            lambda idx_data, key: idx_data[key]['ids'],
+            log_msg="Verification of code documents to index started"
+        )
+
+    def index_documents(self, documents: Generator[Document, None, None], progress_step: int = 20, clean_index: bool = True, is_code: bool = False):
+        """ Index documents in the vectorstore.
+
+        Args:
+            documents (Any): Generator or list of documents to index.
+            progress_step (int): Step for progress reporting, default is 20.
+            clean_index (bool): If True, clean the index before re-indexing all documents.
+        """
+
         from ..langchain.interfaces.llm_processor import add_documents
+
+        # pre-process documents if needed (find duplicates, etc.)
+        if clean_index:
+            logger.info("Cleaning index before re-indexing all documents.")
+            self._log_data("Cleaning index before re-indexing all documents. Previous index will be removed", tool_name="index_documents")
+            try:
+                self._clean_collection()
+                self.vectoradapter.persist()
+                self.vectoradapter.vacuum()
+                self._log_data("Previous index has been removed",
+                               tool_name="index_documents")
+            except Exception as e:
+                logger.warning(f"Failed to clean index: {str(e)}. Continuing with re-indexing.")
+            if isinstance(documents, types.GeneratorType):
+                documents = list(documents)
+        else:
+            # remove duplicates based on metadata 'id' and 'updated_on' or 'commit_hash' fields
+            documents = self._reduce_code_duplicates(documents, self.vectoradapter.vectorstore) if is_code \
+                else self._reduce_non_code_duplicates(documents, self.vectoradapter.vectorstore)
+
+        if not documents or len(documents) == 0:
+            logger.info("No new documents to index after duplicate check.")
+            return {"status": "ok", "message": "No new documents to index."}
+
+        # if func is provided, apply it to documents
+        # used for processing of documents before indexing,
+        # e.g. to avoid time-consuming operations for documents that are already indexed
+        dependent_docs_generator = self.process_document_func(documents) if self.process_document_func else []
+
+        # notify user about missed required metadata fields: id, updated_on
+        # it is not required to have them, but it is recommended to have them for proper re-indexing and duplicate detection
+        for doc in documents:
+            if 'id' not in doc.metadata or 'updated_on' not in doc.metadata:
+                logger.warning(f"Document is missing required metadata field 'id' or 'updated_on': {doc.metadata}")
+
         logger.debug(f"Indexing documents: {documents}")
         logger.debug(self.vectoradapter)
-        self.vectoradapter.delete_dataset(self.dataset)
-        self.vectoradapter.persist()
-        logger.debug(f"Deleted Dataset")
-        #
-        self.vectoradapter.vacuum()
-        #
+
+        documents = documents + list(dependent_docs_generator)
+        total_docs = len(documents)
         documents_count = 0
         _documents = []
+
+        # set default progress step to 20 if out of 0...100 or None
+        progress_step = 20 if progress_step not in range(0, 100) else progress_step
+        next_progress_point = progress_step
         for document in documents:
             documents_count += 1
             # logger.debug(f"Indexing document: {document}")
@@ -203,7 +403,14 @@ class VectorStoreWrapper(BaseToolApiWrapper):
                     add_documents(vectorstore=self.vectoradapter.vectorstore, documents=_documents)
                     self.vectoradapter.persist()
                     _documents = []
-            except Exception as e:
+
+                percent = math.floor((documents_count / total_docs) * 100)
+                if percent >= next_progress_point:
+                    msg = f"Indexing progress: {percent}%. Processed {documents_count} of {total_docs} documents."
+                    logger.debug(msg)
+                    self._log_data(msg)
+                    next_progress_point += progress_step
+            except Exception:
                 from traceback import format_exc
                 logger.error(f"Error: {format_exc()}")
                 return {"status": "error", "message": f"Error: {format_exc()}"}
@@ -329,8 +536,12 @@ class VectorStoreWrapper(BaseToolApiWrapper):
             )
             
         # Initialize document map for tracking by ID
-        doc_map = {doc.metadata.get('id', f"idx_{i}"): (doc, score) 
-                  for i, (doc, score) in enumerate(vector_items)}
+        doc_map = {
+            f"{doc.metadata.get('id', f'idx_{i}')}_{doc.metadata['chunk_id']}"
+            if 'chunk_id' in doc.metadata
+            else doc.metadata.get('id', f"idx_{i}"): (doc, score)
+            for i, (doc, score) in enumerate(vector_items)
+        }
         
         # Process full-text search if configured
         if full_text_search and full_text_search.get('enabled') and full_text_search.get('fields'):
@@ -383,9 +594,11 @@ class VectorStoreWrapper(BaseToolApiWrapper):
             combined_items = [item for item in combined_items if abs(item[1]) >= cut_off]
         
         # Sort by score and limit results
-        combined_items.sort(key=lambda x: x[1], reverse=True)
+
+        # for chroma we want ascending order (lower score is better), for others descending
+        combined_items.sort(key=lambda x: x[1], reverse= self.vectorstore_type.lower() != 'chroma')
         combined_items = combined_items[:search_top]
-        
+
         # Format output based on doctype
         if doctype == 'code':
             return code_format(combined_items)
@@ -498,6 +711,21 @@ class VectorStoreWrapper(BaseToolApiWrapper):
         ])
         return result.content
 
+    def _log_data(self, message: str, tool_name: str = "index_data"):
+        """Log data and dispatch custom event for indexing progress"""
+
+        try:
+            dispatch_custom_event(
+                name="thinking_step",
+                data={
+                    "message": message,
+                    "tool_name": tool_name,
+                    "toolkit": "vectorstore",
+                },
+            )
+        except Exception as e:
+            logger.warning(f"Failed to dispatch progress event: {str(e)}")
+
     def get_available_tools(self):
         return [
             {
