@@ -250,6 +250,7 @@ class ReposApiWrapper(BaseCodeToolApiWrapper):
     token: Optional[SecretStr]
     _client: Optional[GitClient] = PrivateAttr()
 
+    llm: Optional[Any] = None
     # Vector store configuration
     connection_string: Optional[SecretStr] = None
     collection_name: Optional[str] = None
@@ -301,26 +302,69 @@ class ReposApiWrapper(BaseCodeToolApiWrapper):
 
         return values
 
+    def _get_commits(self, file_path: str, branch: str, top: int = None) -> List[GitCommitRef]:
+        """
+        Get commits for a specific file in a specific branch.
+
+        Args:
+            file_path (str): Path to the file in the repository.
+            branch (str): Branch name to get commits from.
+            top (int, optional): Maximum number of commits to return. Defaults to None.
+
+        Returns:
+            List[GitCommitRef]: List of commit references.
+        """
+        try:
+            version_descriptor = GitVersionDescriptor(
+                version=branch, version_type="branch"
+            )
+            commits = self._client.get_commits(
+                repository_id=self.repository_id,
+                project=self.project,
+                search_criteria=GitQueryCommitsCriteria(item_path=file_path,
+                                                       item_version=version_descriptor, top=top if top else 100),
+            )
+            return commits
+        except Exception as e:
+            msg = f"Failed to get commits for file '{file_path}' on branch '{branch}': {str(e)}"
+            logger.error(msg)
+            return ToolException(msg)
+
+    def _file_commit_hash(self, file_path: str, branch: str) -> str:
+        """Get the commit hash of the last commit that modified a file in a specific branch."""
+
+        commits = self._get_commits(file_path, branch, top=1)
+        if commits:
+            return commits[0].commit_id
+        else:
+            return None
+
     def _get_files(
             self,
-            directory_path: str = "",
-            branch_name: str = None,
+            path: str = "",
+            branch: str = None,
             recursion_level: str = "Full",
     ) -> str:
+        """Get list of files from a repository path and branch.
+
+        Args:
+            path (str): Path within the repository to list files from
+            branch (str): Branch to get files from. Defaults to base_branch if None.
+            recursion_level (str): OneLevel - includes immediate children, Full - includes all items, None - no recursion
+
+        Returns:
+            List[str]: List of file paths
         """
-        Params:
-            recursion_level: OneLevel - includes immediate children, Full - includes all items, None - no recursion
-        """
-        branch_name = branch_name if branch_name else self.base_branch
+        branch = branch if branch else self.base_branch
         files: List[str] = []
         try:
             version_descriptor = GitVersionDescriptor(
-                version=branch_name, version_type="branch"
+                version=branch, version_type="branch"
             )
             items = self._client.get_items(
                 repository_id=self.repository_id,
                 project=self.project,
-                scope_path=directory_path,
+                scope_path=path,
                 recursion_level=recursion_level,
                 version_descriptor=version_descriptor,
                 include_content_metadata=True,
@@ -334,7 +378,7 @@ class ReposApiWrapper(BaseCodeToolApiWrapper):
             item = items.pop(0)
             if item.git_object_type == "blob":
                 files.append(item.path)
-        return str(files)
+        return files # Changed to return list directly instead of str
 
     def set_active_branch(self, branch_name: str) -> str:
         """
@@ -389,7 +433,7 @@ class ReposApiWrapper(BaseCodeToolApiWrapper):
             logger.error(msg)
             return ToolException(msg)
 
-    def list_files(self, directory_path: str = "", branch_name: str = None) -> str:
+    def list_files(self, directory_path: str = "", branch_name: str = None) -> List[str]:
         """
         Recursively fetches files from a directory in the repo.
 
@@ -398,12 +442,12 @@ class ReposApiWrapper(BaseCodeToolApiWrapper):
             branch_name (str): The name of the branch where the files to be received.
 
         Returns:
-            str: List of file paths, or an error message.
+            List[str]: List of file paths, or an error message.
         """
         self.active_branch = branch_name if branch_name else self.active_branch
         return self._get_files(
-            directory_path=directory_path,
-            branch_name=self.active_branch if self.active_branch else self.base_branch,
+            path=directory_path,
+            branch=self.active_branch if self.active_branch else self.base_branch,
         )
 
     def parse_pull_request_comments(
@@ -1106,7 +1150,8 @@ class ReposApiWrapper(BaseCodeToolApiWrapper):
         """
         try:
             search_criteria = GitQueryCommitsCriteria(
-                item_version=GitVersionDescriptor(version=sha, version_type='commit'),
+                # get from the active branch if not specified
+                item_version=GitVersionDescriptor(version=sha, version_type='commit') if sha else None,
                 item_path=path,
                 from_date=str(datetime.fromisoformat(since)) if since else None,
                 to_date=str(datetime.fromisoformat(until)) if until else None,
