@@ -1,19 +1,25 @@
 import json
 import logging
-from typing import Optional
+import xml.etree.ElementTree as ET
+from typing import Generator, Optional
 
 from azure.devops.connection import Connection
 from azure.devops.v7_0.test_plan.models import TestPlanCreateParams, TestSuiteCreateParams, \
     SuiteTestCaseCreateUpdateParameters
 from azure.devops.v7_0.test_plan.test_plan_client import TestPlanClient
+from langchain_core.documents import Document
 from langchain_core.tools import ToolException
 from msrest.authentication import BasicAuthentication
 from pydantic import create_model, PrivateAttr, model_validator, SecretStr
 from pydantic.fields import FieldInfo as Field
-import xml.etree.ElementTree as ET
 
 from ..work_item import AzureDevOpsApiWrapper
-from ...elitea_base import BaseToolApiWrapper
+from ...elitea_base import BaseVectorStoreToolApiWrapper, extend_with_vector_tools
+
+try:
+    from alita_sdk.runtime.langchain.interfaces.llm_processor import get_embeddings
+except ImportError:
+    from alita_sdk.langchain.interfaces.llm_processor import get_embeddings
 
 logger = logging.getLogger(__name__)
 
@@ -96,7 +102,6 @@ TestCaseAddModel = create_model(
     suite_id=(int, Field(description="ID of the test suite to which test cases are to be added"))
 )
 
-
 test_steps_description = """Json or XML array string with test steps. 
     Json example: [{"stepNumber": 1, "action": "Some action", "expectedResult": "Some expectation"},...]
     XML example: 
@@ -158,7 +163,7 @@ TestCasesGetModel = create_model(
     suite_id=(int, Field(description="ID of the test suite for which test cases are requested"))
 )
 
-class TestPlanApiWrapper(BaseToolApiWrapper):
+class TestPlanApiWrapper(BaseVectorStoreToolApiWrapper):
     __test__ = False
     organization_url: str
     project: str
@@ -250,8 +255,10 @@ class TestPlanApiWrapper(BaseToolApiWrapper):
         try:
             if isinstance(suite_test_case_create_update_parameters, str):
                 suite_test_case_create_update_parameters = json.loads(suite_test_case_create_update_parameters)
-            suite_test_case_create_update_params_obj = [SuiteTestCaseCreateUpdateParameters(**param) for param in suite_test_case_create_update_parameters]
-            test_cases = self._client.add_test_cases_to_suite(suite_test_case_create_update_params_obj, self.project, plan_id, suite_id)
+            suite_test_case_create_update_params_obj = [SuiteTestCaseCreateUpdateParameters(**param) for param in
+                                                        suite_test_case_create_update_parameters]
+            test_cases = self._client.add_test_cases_to_suite(suite_test_case_create_update_params_obj, self.project,
+                                                              plan_id, suite_id)
             return [test_case.as_dict() for test_case in test_cases]
         except Exception as e:
             logger.error(f"Error adding test case: {e}")
@@ -268,10 +275,11 @@ class TestPlanApiWrapper(BaseToolApiWrapper):
             test_steps=test_case['test_steps'],
             test_steps_format=test_case['test_steps_format']) for test_case in test_cases]
 
-
-    def create_test_case(self, plan_id: int, suite_id: int, title: str, description: str, test_steps: str, test_steps_format: str = 'json'):
+    def create_test_case(self, plan_id: int, suite_id: int, title: str, description: str, test_steps: str,
+                         test_steps_format: str = 'json'):
         """Creates a new test case in specified suite in Azure DevOps."""
-        work_item_wrapper = AzureDevOpsApiWrapper(organization_url=self.organization_url, token=self.token.get_secret_value(), project=self.project)
+        work_item_wrapper = AzureDevOpsApiWrapper(organization_url=self.organization_url,
+                                                  token=self.token.get_secret_value(), project=self.project)
         if test_steps_format == 'json':
             steps_xml = self.get_test_steps_xml(json.loads(test_steps))
         elif test_steps_format == 'xml':
@@ -279,8 +287,9 @@ class TestPlanApiWrapper(BaseToolApiWrapper):
         else:
             return ToolException("Unknown test steps format: " + test_steps_format)
         work_item_json = self.build_ado_test_case(title, description, steps_xml)
-        created_work_item_id = work_item_wrapper.create_work_item(work_item_json=json.dumps(work_item_json), wi_type="Test Case")['id']
-        return self.add_test_case([{"work_item":{"id":created_work_item_id}}], plan_id, suite_id)
+        created_work_item_id = \
+        work_item_wrapper.create_work_item(work_item_json=json.dumps(work_item_json), wi_type="Test Case")['id']
+        return self.add_test_case([{"work_item": {"id": created_work_item_id}}], plan_id, suite_id)
 
     def build_ado_test_case(self, title, description, steps_xml):
         """
@@ -355,6 +364,37 @@ class TestPlanApiWrapper(BaseToolApiWrapper):
             logger.error(f"Error getting test cases: {e}")
             return ToolException(f"Error getting test cases: {e}")
 
+    def _base_loader(self, plan_id: str, suite_ids: Optional[list[str]] = []) -> Generator[Document, None, None]:
+        cases = []
+        for sid in suite_ids:
+            cases.extend(self.get_test_cases(plan_id, sid))
+        #
+        for case in cases:
+            field_dicts = case.get('work_item', {}).get('work_item_fields', [])
+            data = {k: v for d in field_dicts for k, v in d.items()}
+            yield Document(
+                page_content=data.get('Microsoft.VSTS.TCM.Steps', ''),
+                metadata={
+                    'id': case.get('work_item', {}).get('id', ''),
+                    'title': case.get('work_item', {}).get('name', ''),
+                    'plan_id': case.get('test_plan', {}).get('id', ''),
+                    'suite_id': case.get('test_suite', {}).get('id', ''),
+                    'description': data.get('System.Description', ''),
+                    'updated_on': data.get('System.Rev', ''),
+                })
+
+    def _process_document(self, document: Document) -> Generator[Document, None, None]:
+        if False:
+            yield  # Unreachable, but keeps the function a generator
+
+    def _index_tool_params(self):
+        """Return the parameters for indexing data."""
+        return {
+            "plan_id": (str, Field(description="ID of the test plan for which test cases are requested")),
+            "suite_ids": (str, Field(description="List of test suite IDs for which test cases are requested (can be empty)"))
+        }
+
+    @extend_with_vector_tools
     def get_available_tools(self):
         """Return a list of available tools."""
         return [
