@@ -1,4 +1,3 @@
-import shortuuid
 import re
 import logging
 import requests
@@ -17,7 +16,7 @@ from langchain_core.messages import HumanMessage
 from markdownify import markdownify
 from langchain_community.document_loaders.confluence import ContentFormat
 
-from ..elitea_base import BaseVectorStoreToolApiWrapper, BaseIndexParams
+from ..elitea_base import BaseVectorStoreToolApiWrapper, extend_with_vector_tools
 from ..llm.img_utils import ImageDescriptionCache
 from ..utils import is_cookie_token, parse_cookie_string
 
@@ -141,50 +140,6 @@ pageId = create_model(
                                  description="Optional JSON of parameters to be sent in request body or query params. MUST be string with valid JSON. For search/read operations, you MUST always get minimum fields and set max results, until users ask explicitly for more fields. For search/read operations you must generate CQL query string and pass it as params."))
 )
 
-# loaderParams = create_model(
-#     "LoaderParams",
-#     content_format=(str, Field(description="The format of the content to be retrieved.")),
-#     page_ids=(Optional[List[str]], Field(description="List of page IDs to retrieve.", default=None)),
-#     label=(Optional[str], Field(description="Label to filter pages.", default=None)),
-#     cql=(Optional[str], Field(description="CQL query to filter pages.", default=None)),
-#     include_restricted_content=(Optional[bool], Field(description="Include restricted content.", default=False)),
-#     include_archived_content=(Optional[bool], Field(description="Include archived content.", default=False)),
-#     include_attachments=(Optional[bool], Field(description="Include attachments.", default=False)),
-#     include_comments=(Optional[bool], Field(description="Include comments.", default=False)),
-#     include_labels=(Optional[bool], Field(description="Include labels.", default=False)),
-#     limit=(Optional[int], Field(description="Limit the number of results.", default=10)),
-#     max_pages=(Optional[int], Field(description="Maximum number of pages to retrieve.", default=1000)),
-#     ocr_languages=(Optional[str], Field(description="OCR languages for processing attachments.", default=None)),
-#     keep_markdown_format=(Optional[bool], Field(description="Keep the markdown format.", default=True)),
-#     keep_newlines=(Optional[bool], Field(description="Keep newlines in the content.", default=True)),
-#     bins_with_llm=(Optional[bool], Field(description="Use LLM for processing binary files.", default=False)),
-# )
-
-indexPagesParams = create_model(
-    "indexPagesParams",
-    __base__=BaseIndexParams,
-    content_format=(Literal['view', 'storage', 'export_view', 'editor', 'anonymous'],
-                    Field(description="The format of the content to be retrieved.")),
-    ### Loader Parameters
-    page_ids=(Optional[List[str]], Field(description="List of page IDs to retrieve.", default=None)),
-    label=(Optional[str], Field(description="Label to filter pages.", default=None)),
-    cql=(Optional[str], Field(description="CQL query to filter pages.", default=None)),
-    limit=(Optional[int], Field(description="Limit the number of results.", default=10)),
-    max_pages=(Optional[int], Field(description="Maximum number of pages to retrieve.", default=1000)),
-    include_restricted_content=(Optional[bool], Field(description="Include restricted content.", default=False)),
-    include_archived_content=(Optional[bool], Field(description="Include archived content.", default=False)),
-    include_attachments=(Optional[bool], Field(description="Include attachments.", default=False)),
-    include_comments=(Optional[bool], Field(description="Include comments.", default=False)),
-    include_labels=(Optional[bool], Field(description="Include labels.", default=True)),
-    ocr_languages=(Optional[str], Field(description="OCR languages for processing attachments.", default='eng')),
-    keep_markdown_format=(Optional[bool], Field(description="Keep the markdown format.", default=True)),
-    keep_newlines=(Optional[bool], Field(description="Keep newlines in the content.", default=True)),
-    bins_with_llm=(Optional[bool], Field(description="Use LLM for processing binary files.", default=False)),
-    ### Chunking Parameters
-    chunking_tool=(Literal['markdown', 'statistical', 'proposal'], Field(description="Name of chunking tool", default="markdown")),
-    chunking_config=(Optional[dict], Field(description="Chunking tool configuration", default_factory=dict)),
-)
-
 GetPageWithImageDescriptions = create_model(
     "GetPageWithImageDescriptionsModel",
     page_id=(str, Field(description="Confluence page ID from which content with images will be extracted")),
@@ -201,6 +156,8 @@ GetPageAttachmentsInput = create_model(
     page_id=(str, Field(description="Confluence page ID from which attachments will be retrieved")),
     max_content_length=(int, Field(default=10000, description="Maximum number of characters to return for attachment content. Content will be truncated if longer. Default is 10000.")),
     custom_prompt=(Optional[str], Field(default=None, description="Custom prompt to use for LLM-based analysis of attachments (images, pdfs, etc). If not provided, a default prompt will be used.")),
+    allowed_extensions=(Optional[List[str]], Field(default=None, description="List of file extensions to include (e.g. ['pdf', 'docx']). If None, all extensions are included.", examples=[["pdf", "docx"]])),
+    name_pattern=(Optional[str], Field(default=None, description="Regex pattern to filter attachment names (e.g. '^report_.*\\.pdf$'). If None, all names are included.", examples=["^report_.*\\.pdf$"])),
 )
 
 
@@ -847,7 +804,7 @@ class ConfluenceAPIWrapper(BaseVectorStoreToolApiWrapper):
             docs.extend(batch)
         return docs[:max_pages]
 
-    def _loader(self, **kwargs) -> Generator[str, None, None]:
+    def _base_loader(self, **kwargs) -> Generator[Document, None, None]:
         """
         Loads content from Confluence based on parameters.
         Returns:
@@ -856,7 +813,15 @@ class ConfluenceAPIWrapper(BaseVectorStoreToolApiWrapper):
         from .loader import AlitaConfluenceLoader
         from copy import copy
         content_format = kwargs.get('content_format', 'view').lower()
+        base_params = {
+            'url': self.base_url,
+            'space_key': self.space,
+            'min_retry_seconds': self.min_retry_seconds,
+            'max_retry_seconds': self.max_retry_seconds,
+            'number_of_retries': self.number_of_retries
+        }
         confluence_loader_params = copy(kwargs)
+        confluence_loader_params.update(base_params)
         mapping = {
             'view': ContentFormat.VIEW,
             'storage': ContentFormat.STORAGE,
@@ -874,88 +839,13 @@ class ConfluenceAPIWrapper(BaseVectorStoreToolApiWrapper):
         loader = AlitaConfluenceLoader(self.client, self.llm, bins_with_llm, **confluence_loader_params)
 
         for document in loader._lazy_load(kwargs={}):
+            if 'updated_on' not in document.metadata and 'when' in document.metadata:
+                document.metadata['updated_on'] = document.metadata['when']
             yield document
 
-    def index_data(self, content_format: str,
-                    collection_suffix: str = "",
-                    page_ids: Optional[List[str]] = None,
-                    label: Optional[str] = None,
-                    cql: Optional[str] = None,
-                    include_restricted_content: Optional[bool] = False,
-                    include_archived_content: Optional[bool] = False,
-                    include_attachments: Optional[bool] = False,
-                    include_comments: Optional[bool] = False,
-                    include_labels: Optional[bool] = False,
-                    limit: Optional[int] = 10,
-                    max_pages: Optional[int] = 10,
-                    keep_markdown_format: Optional[bool] = True,
-                    keep_newlines: Optional[bool] = True,
-                    bins_with_llm: bool = False,
-                    chunking_tool: str = "markdown",
-                    chunking_config: Optional[Dict[str, Any]] = None,
-                    **kwargs) -> Generator[str, None, None]:
-        """Load Confluence pages and index them in the vector store."""
-
-        from alita_sdk.tools.chunkers import __confluence_chunkers__ as chunkers, __confluence_models__ as models
-        try:
-            from alita_sdk.langchain.interfaces.llm_processor import get_embeddings
-        except ImportError:
-            from alita_sdk.runtime.langchain.interfaces.llm_processor import get_embeddings
-
-        loader_params = {
-            'url': self.base_url,
-            'space_key': self.space,
-            'content_format': content_format,
-            'page_ids': page_ids,
-            'label': label,
-            'cql': cql,
-            'include_restricted_content': include_restricted_content,
-            'include_archived_content': include_archived_content,
-            'include_attachments': include_attachments,
-            'include_comments': include_comments,
-            'include_labels': include_labels,
-            'limit': limit,
-            'max_pages': max_pages,
-            'keep_markdown_format': keep_markdown_format,
-            'keep_newlines': keep_newlines,
-            'bins_with_llm': bins_with_llm,
-            'min_retry_seconds': self.min_retry_seconds,
-            'max_retry_seconds': self.max_retry_seconds,
-            'number_of_retries': self.number_of_retries
-        }
-        documents = self._loader(**loader_params)
-        embedding = get_embeddings(self.embedding_model, self.embedding_model_params)
-
-        chunker = chunkers.get(chunking_tool)
-
-        chunking_config = chunking_config or {}
-
-        if chunker:
-            # Validate and prepare chunking configuration using Pydantic models
-            config_model = models.get(chunking_tool)
-            if config_model:
-                # Set required fields that should come from the instance
-                chunking_config['embedding'] = embedding
-                chunking_config['llm'] = self.llm
-
-                try:
-                    # Validate the configuration using the appropriate Pydantic model
-                    validated_config = config_model(**chunking_config)
-                    chunking_config = validated_config.model_dump()
-                except Exception as e:
-                    logger.error(f"Invalid chunking configuration for {chunking_tool}: {e}")
-                    raise ToolException(f"Invalid chunking configuration: {e}")
-            else:
-                # Fallback for chunkers without models
-                chunking_config['embedding'] = embedding
-                chunking_config['llm'] = self.llm
-
-            documents = chunker(documents, chunking_config)
-
-        # passing embedding to avoid re-initialization
-        vectorstore = self._init_vector_store(collection_suffix, embeddings=embedding)
-        return vectorstore.index_documents(documents)
-
+    def _process_document(self, document: Document) -> Generator[Document, None, None]:
+        for attachment in self.get_page_attachments(document.metadata.get('id')):
+            yield Document(page_content=attachment.get('content', ''), metadata=attachment.get('metadata', {}))
 
     def _download_image(self, image_url):
         """
@@ -1427,8 +1317,8 @@ class ConfluenceAPIWrapper(BaseVectorStoreToolApiWrapper):
             stacktrace = traceback.format_exc()
             logger.error(f"Error processing page with images: {stacktrace}")
             return f"Error processing page with images: {str(e)}"
-
-    def get_page_attachments(self, page_id: str, max_content_length: int = 10000, custom_prompt: str = None):
+    
+    def get_page_attachments(self, page_id: str, max_content_length: int = 10000, custom_prompt: str = None, allowed_extensions: Optional[List[str]] = None, name_pattern: Optional[str] = None):
         """
         Retrieve all attachments for a Confluence page, including core metadata (with creator, created, updated), comments,
         file content, and LLM-based analysis for supported types.
@@ -1449,8 +1339,19 @@ class ConfluenceAPIWrapper(BaseVectorStoreToolApiWrapper):
                     logger.warning(f"Failed to fetch history for attachment {attachment.get('title', '')}: {str(e)}")
                     history_map[attachment['id']] = None
 
+            import re
             results = []
             for attachment in attachments['results']:
+                title = attachment.get('title', '')
+                file_ext = title.lower().split('.')[-1] if '.' in title else ''
+
+                # Filter by allowed_extensions
+                if allowed_extensions and file_ext not in allowed_extensions:
+                    continue
+                # Filter by name_pattern
+                if name_pattern and not re.match(name_pattern, title):
+                    continue
+
                 media_type = attachment.get('metadata', {}).get('mediaType', '')
                 # Core metadata extraction with history
                 hist = history_map.get(attachment['id']) or {}
@@ -1458,7 +1359,7 @@ class ConfluenceAPIWrapper(BaseVectorStoreToolApiWrapper):
                 created_date = hist.get('createdDate', '') if hist else attachment.get('created', '')
                 last_updated = hist.get('lastUpdated', {}).get('when', '') if hist else ''
                 metadata = {
-                    'name': attachment.get('title', ''),
+                    'name': title,
                     'size': attachment.get('extensions', {}).get('fileSize', None),
                     'creator': created_by,
                     'created': created_date,
@@ -1467,6 +1368,7 @@ class ConfluenceAPIWrapper(BaseVectorStoreToolApiWrapper):
                     'labels': [label['name'] for label in attachment.get('metadata', {}).get('labels', {}).get('results', [])],
                     'download_url': self.base_url.rstrip('/') + attachment['_links']['download'] if attachment.get('_links', {}).get('download') else None
                 }
+
                 # Fetch comments for the attachment
                 comments = []
                 try:
@@ -1480,16 +1382,13 @@ class ConfluenceAPIWrapper(BaseVectorStoreToolApiWrapper):
                                 'body': comment.get('body', {}).get('storage', {}).get('value', '')
                             })
                 except Exception as e:
-                    logger.warning(f"Failed to fetch comments for attachment {attachment.get('title', '')}: {str(e)}")
+                    logger.warning(f"Failed to fetch comments for attachment {title}: {str(e)}")
 
                 content = None
                 llm_analysis = None
-                title = attachment.get('title', '')
                 download_url = self.base_url.rstrip('/') + attachment['_links']['download']
 
                 # --- Begin: Raw content for xml, json, markdown, txt ---
-                # Check by media type or file extension
-                file_ext = title.lower().split('.')[-1] if '.' in title else ''
                 is_text_type = (
                     media_type in [
                         'application/xml', 'text/xml',
@@ -1661,7 +1560,7 @@ class ConfluenceAPIWrapper(BaseVectorStoreToolApiWrapper):
                             llm_analysis = self._process_image_with_llm(image_data, title, context_text, custom_prompt)
 
                 if llm_analysis and isinstance(llm_analysis, str) and len(llm_analysis) > max_content_length:
-                        llm_analysis = llm_analysis[:max_content_length] + f"\n...[truncated, showing first {max_content_length} characters]"
+                    llm_analysis = llm_analysis[:max_content_length] + f"\n...[truncated, showing first {max_content_length} characters]"
 
                 results.append({
                     'metadata': metadata,
@@ -1674,6 +1573,28 @@ class ConfluenceAPIWrapper(BaseVectorStoreToolApiWrapper):
             logger.error(f"Error retrieving attachments for page {page_id}: {str(e)}")
             return f"Error retrieving attachments: {str(e)}"
 
+    def _index_tool_params(self):
+        """Return the parameters for indexing data."""
+        return {
+            "content_format": (Literal['view', 'storage', 'export_view', 'editor', 'anonymous'],
+                            Field(description="The format of the content to be retrieved.")),
+            "page_ids": (Optional[List[str]], Field(description="List of page IDs to retrieve.", default=None)),
+            "label": (Optional[str], Field(description="Label to filter pages.", default=None)),
+            "cql": (Optional[str], Field(description="CQL query to filter pages.", default=None)),
+            "limit": (Optional[int], Field(description="Limit the number of results.", default=10)),
+            "max_pages": (Optional[int], Field(description="Maximum number of pages to retrieve.", default=1000)),
+            "include_restricted_content": (Optional[bool], Field(description="Include restricted content.", default=False)),
+            "include_archived_content": (Optional[bool], Field(description="Include archived content.", default=False)),
+            "include_attachments": (Optional[bool], Field(description="Include attachments.", default=False)),
+            "include_comments": (Optional[bool], Field(description="Include comments.", default=False)),
+            "include_labels": (Optional[bool], Field(description="Include labels.", default=True)),
+            "ocr_languages": (Optional[str], Field(description="OCR languages for processing attachments.", default='eng')),
+            "keep_markdown_format": (Optional[bool], Field(description="Keep the markdown format.", default=True)),
+            "keep_newlines": (Optional[bool], Field(description="Keep newlines in the content.", default=True)),
+            "bins_with_llm": (Optional[bool], Field(description="Use LLM for processing binary files.", default=False)),
+        }
+
+    @extend_with_vector_tools
     def get_available_tools(self):
         # Confluence-specific tools
         confluence_tools = [
@@ -1785,13 +1706,6 @@ class ConfluenceAPIWrapper(BaseVectorStoreToolApiWrapper):
                 "description": self.get_page_id_by_title.__doc__,
                 "args_schema": getPageIdByTitleInput,
             },
-            # Confluence-specific vector store indexing
-            {
-                "name": "index_data",
-                "ref": self.index_data,
-                "description": self.index_data.__doc__,
-                "args_schema": indexPagesParams,
-            },
             {
                 "name": "get_page_attachments",
                 "ref": self.get_page_attachments,
