@@ -2,10 +2,16 @@ import ast
 import fnmatch
 import logging
 import traceback
-from typing import Any, Optional, List, Dict
+from typing import Any, Optional, List, Literal, Dict, Generator
+
+from langchain_core.documents import Document
 from langchain_core.tools import ToolException
-from pydantic import BaseModel, create_model, Field
+from pydantic import BaseModel, create_model, Field, SecretStr
+
+from alita_sdk.runtime.langchain.interfaces.llm_processor import get_embeddings
+from .chunkers import markdown_chunker
 from .utils import TOOLKIT_SPLITTER
+from ..runtime.utils.utils import IndexerKeywords
 
 logger = logging.getLogger(__name__)
 
@@ -35,6 +41,11 @@ BaseCodeIndexParams = create_model(
     blacklist=(Optional[List[str]], Field(description="File extensions or paths to exclude. Defaults to no exclusions if None.", default=None)),
 )
 
+RemoveIndexParams = create_model(
+    "RemoveIndexParams",
+    collection_suffix=(Optional[str], Field(description="Optional suffix for collection name (max 7 characters)", default="", max_length=7)),
+)
+
 BaseSearchParams = create_model(
     "BaseSearchParams",
     query=(str, Field(description="Query text to search in the index")),
@@ -96,6 +107,17 @@ BaseStepbackSearchParams = create_model(
     )),
 )
 
+BaseIndexDataParams = create_model(
+    "indexData",
+    __base__=BaseIndexParams,
+    progress_step=(Optional[int], Field(default=None, ge=0, le=100,
+                         description="Optional step size for progress reporting during indexing")),
+    clean_index=(Optional[bool], Field(default=False,
+                       description="Optional flag to enforce clean existing index before indexing new data")),
+    chunking_tool=(Literal['','markdown', 'statistical', 'proposal'], Field(description="Name of chunking tool", default=None)),
+    chunking_config=(Optional[dict], Field(description="Chunking tool configuration", default_factory=dict)),
+)
+
 
 class BaseToolApiWrapper(BaseModel):
 
@@ -186,7 +208,124 @@ class BaseToolApiWrapper(BaseModel):
 
 class BaseVectorStoreToolApiWrapper(BaseToolApiWrapper):
     """Base class for tool API wrappers that support vector store functionality."""
-    
+
+    doctype: str = "document"
+
+    llm: Any = None
+    connection_string: Optional[SecretStr] = None
+    collection_name: Optional[str] = None
+    embedding_model: Optional[str] = "HuggingFaceEmbeddings"
+    embedding_model_params: Optional[Dict[str, Any]] = {"model_name": "sentence-transformers/all-MiniLM-L6-v2"}
+    vectorstore_type: Optional[str] = "PGVector"
+
+    def _index_tool_params(self, **kwargs) -> dict[str, tuple[type, Field]]:
+        """
+        Returns a list of fields for index_data args schema.
+        NOTE: override this method in subclasses to provide specific parameters for certain toolkit.
+        """
+        return {}
+
+    def _get_dependencies_chunker(self, document: Optional[Document] = None):
+        return markdown_chunker
+
+    def _get_dependencies_chunker_config(self, document: Optional[Document] = None):
+        embedding = get_embeddings(self.embedding_model, self.embedding_model_params)
+        #
+        return {'embedding': embedding, 'llm': self.llm}
+
+    def _base_loader(self, **kwargs) -> Generator[Document, None, None]:
+        """ Loads documents from a source, processes them,
+        and returns a list of Document objects with base metadata: id and created_on."""
+        pass
+
+    def _process_document(self, base_document: Document) -> Generator[Document, None, None]:
+        """ Process an existing base document to extract relevant metadata for full document preparation.
+        Used for late processing of documents after we ensure that the document has to be indexed to avoid
+        time-consuming operations for documents which might be useless.
+
+        Args:
+            document (Document): The base document to process.
+
+        Returns:
+            Document: The processed document with metadata."""
+        pass
+
+    def get_index_data_tool(self):
+        return {
+            "name": "index_data",
+            "ref": self.index_data,
+            "description": "Loads data to index.",
+            "args_schema": create_model(
+                "IndexData",
+                __base__=BaseIndexDataParams,
+                **self._index_tool_params() if self._index_tool_params() else {}
+            )
+        }
+
+    def index_data(self, **kwargs):
+        from alita_sdk.tools.chunkers import __confluence_chunkers__ as chunkers, __confluence_models__ as models
+        docs = self._base_loader(**kwargs)
+        embedding = get_embeddings(self.embedding_model, self.embedding_model_params)
+        chunking_tool = kwargs.get("chunking_tool")
+        if chunking_tool:
+            # Resolve chunker from the provided chunking_tool name
+            base_chunker = chunkers.get(chunking_tool)
+            # Resolve chunking configuration
+            base_chunking_config = kwargs.get("chunking_config", {})
+            config_model = models.get(chunking_tool)
+            # Set required fields that should come from the instance (and Fallback for chunkers without models)
+            base_chunking_config['embedding'] = embedding
+            base_chunking_config['llm'] = self.llm
+            #
+            if config_model:
+                try:
+                    # Validate the configuration using the appropriate Pydantic model
+                    validated_config = config_model(**base_chunking_config)
+                    base_chunking_config = validated_config.model_dump()
+                except Exception as e:
+                    logger.error(f"Invalid chunking configuration for {chunking_tool}: {e}")
+                    raise ToolException(f"Invalid chunking configuration: {e}")
+            #
+            docs = base_chunker(file_content_generator=docs, config=base_chunking_config)
+        #
+        collection_suffix = kwargs.get("collection_suffix")
+        progress_step = kwargs.get("progress_step")
+        clean_index = kwargs.get("clean_index")
+        vs = self._init_vector_store(collection_suffix, embeddings=embedding)
+        #
+        return vs.index_documents(docs, progress_step=progress_step, clean_index=clean_index)
+
+    def _process_documents(self, documents: List[Document]) -> Generator[Document, None, None]:
+        """
+        Process a list of base documents to extract relevant metadata for full document preparation.
+        Used for late processing of documents after we ensure that the documents have to be indexed to avoid
+        time-consuming operations for documents which might be useless.
+        This function passed to index_documents method of vector store and called after _reduce_duplicates method.
+
+        Args:
+            documents (List[Document]): The base documents to process.
+
+        Returns:
+            Generator[Document, None, None]: A generator yielding processed documents with metadata.
+        """
+        for doc in documents:
+            # Filter documents to process only those that either:
+            # - do not have a 'chunk_id' in their metadata, or
+            # - have 'chunk_id' explicitly set to 1.
+            # This prevents processing of irrelevant or duplicate chunks, improving efficiency.
+            chunk_id = doc.metadata.get("chunk_id")
+            if chunk_id is None or chunk_id == 1:
+                processed_docs = self._process_document(doc)
+                if processed_docs:  # Only proceed if the list is not empty
+                    for processed_doc in processed_docs:
+                        # map processed document (child) to the original document (parent)
+                        processed_doc.metadata[IndexerKeywords.PARENT.value] = doc.metadata.get('id', None)
+                        if chunker:=self._get_dependencies_chunker(processed_doc):
+                            yield from chunker(file_content_generator=iter([processed_doc]), config=self._get_dependencies_chunker_config())
+                        else:
+                            yield processed_doc
+
+
     def _init_vector_store(self, collection_suffix: str = "", embeddings: Optional[Any] = None):
         """ Initializes the vector store wrapper with the provided parameters."""
         try:
@@ -225,9 +364,17 @@ class BaseVectorStoreToolApiWrapper(BaseToolApiWrapper):
             embedding_model=self.embedding_model,
             embedding_model_params=self.embedding_model_params,
             vectorstore_params=vectorstore_params,
-            embeddings=embeddings
+            embeddings=embeddings,
+            process_document_func=self._process_documents,
         )
 
+    def remove_index(self, collection_suffix: str = ""):
+        """
+            Cleans the indexed data in the collection
+        """
+
+        self._init_vector_store(collection_suffix)._clean_collection()
+
     def search_index(self,
                      query: str,
                      collection_suffix: str = "",
@@ -328,7 +475,14 @@ class BaseVectorStoreToolApiWrapper(BaseToolApiWrapper):
                 "ref": self.stepback_summary_index,
                 "description": self.stepback_summary_index.__doc__,
                 "args_schema": BaseStepbackSearchParams
-            }
+            },
+            {
+                "name": "remove_index",
+                "mode": "remove_index",
+                "ref": self.remove_index,
+                "description": self.remove_index.__doc__,
+                "args_schema": RemoveIndexParams
+            },
         ]
 
 
@@ -340,12 +494,15 @@ class BaseCodeToolApiWrapper(BaseVectorStoreToolApiWrapper):
     def _read_file(self, file_path: str, branch: str):
         raise NotImplementedError("Subclasses should implement this method")
 
+    def _file_commit_hash(self, file_path: str, branch: str):
+        pass
+
     def __handle_get_files(self, path: str, branch: str):
         """
         Handles the retrieval of files from a specific path and branch.
         This method should be implemented in subclasses to provide the actual file retrieval logic.
         """
-        _files = self._get_files(path, branch)
+        _files = self._get_files(path=path, branch=branch)
         if isinstance(_files, str):
             try:
                 # Attempt to convert the string to a list using ast.literal_eval
@@ -405,7 +562,8 @@ class BaseCodeToolApiWrapper(BaseVectorStoreToolApiWrapper):
             for file in _files:
                 if is_whitelisted(file) and not is_blacklisted(file):
                     yield {"file_name": file,
-                           "file_content": self._read_file(file, branch=branch or self.active_branch)}
+                           "file_content": self._read_file(file, branch=branch or self.active_branch),
+                           "commit_hash": self._file_commit_hash(file, branch=branch or self.active_branch)}
 
         return parse_code_files_for_db(file_content_generator())
     
@@ -425,7 +583,7 @@ class BaseCodeToolApiWrapper(BaseVectorStoreToolApiWrapper):
             blacklist=blacklist
         )
         vectorstore = self._init_vector_store(collection_suffix)
-        return vectorstore.index_documents(documents)
+        return vectorstore.index_documents(documents, clean_index=False, is_code=True)
 
     def _get_vector_search_tools(self):
         """
@@ -447,4 +605,16 @@ class BaseCodeToolApiWrapper(BaseVectorStoreToolApiWrapper):
         }
         
         # Return index_data tool first, then the search tools
-        return [index_tool] + base_tools
\ No newline at end of file
+        return [index_tool] + base_tools
+
+def extend_with_vector_tools(method):
+    def wrapper(self, *args, **kwargs):
+        tools = method(self, *args, **kwargs)
+        tools.extend(self._get_vector_search_tools())        
+        #
+        if isinstance(self, BaseVectorStoreToolApiWrapper):
+            tools.append(self.get_index_data_tool())
+        #
+        return tools
+
+    return wrapper
\ No newline at end of file
