@@ -1,14 +1,25 @@
 import json
 import logging
-from typing import Dict, List, Optional, Union
+from typing import Dict, List, Optional, Union, Any, Generator
 
 import pandas as pd
 from langchain_core.tools import ToolException
+from openai import BadRequestError
 from pydantic import SecretStr, create_model, model_validator
 from pydantic.fields import Field, PrivateAttr
 from testrail_api import StatusCodeError, TestRailAPI
 
-from ..elitea_base import BaseToolApiWrapper
+from ..chunkers.code.constants import get_file_extension
+from ..elitea_base import BaseVectorStoreToolApiWrapper, extend_with_vector_tools
+from langchain_core.documents import Document
+
+from ...runtime.utils.utils import IndexerKeywords
+from ..utils.content_parser import parse_file_content
+
+try:
+    from alita_sdk.runtime.langchain.interfaces.llm_processor import get_embeddings
+except ImportError:
+    from alita_sdk.langchain.interfaces.llm_processor import get_embeddings
 
 logger = logging.getLogger(__name__)
 
@@ -281,7 +292,6 @@ updateCase = create_model(
     ),
 )
 
-
 SUPPORTED_KEYS = {
     "id", "title", "section_id", "template_id", "type_id", "priority_id", "milestone_id",
     "refs", "created_by", "created_on", "updated_by", "updated_on", "estimate",
@@ -291,7 +301,7 @@ SUPPORTED_KEYS = {
 }
 
 
-class TestrailAPIWrapper(BaseToolApiWrapper):
+class TestrailAPIWrapper(BaseVectorStoreToolApiWrapper):
     url: str
     password: Optional[SecretStr] = None,
     email: Optional[str] = None,
@@ -461,7 +471,8 @@ class TestrailAPIWrapper(BaseToolApiWrapper):
                 project_id=project_id, **params
             )
 
-            cases = extracted_cases.get("cases")
+            # support old versions of testrail_api
+            cases = extracted_cases.get("cases") if isinstance(extracted_cases, dict) else extracted_cases
 
             if cases is None:
                 return ToolException("No test cases found in the extracted data.")
@@ -492,7 +503,7 @@ class TestrailAPIWrapper(BaseToolApiWrapper):
         you can submit and update specific fields only).
 
         :param case_id: T
-            he ID of the test case
+            He ID of the test case
         :param kwargs:
             :key title: str
                 The title of the test case
@@ -522,6 +533,134 @@ class TestrailAPIWrapper(BaseToolApiWrapper):
             f"Test case #{case_id} has been updated at '{updated_case['updated_on']}')"
         )
 
+    def _base_loader(self, project_id: str,
+                     suite_id: Optional[str] = None,
+                     section_id: Optional[int] = None,
+                     title_keyword: Optional[str] = None,
+                     **kwargs: Any
+                     ) -> Generator[Document, None, None]:
+        self._include_attachments = kwargs.get('include_attachments', False)
+        self._skip_attachment_extensions = kwargs.get('skip_attachment_extensions', [])
+
+        try:
+            if suite_id:
+                resp = self._client.cases.get_cases(project_id=project_id, suite_id=int(suite_id))
+                cases = resp.get('cases', [])
+            else:
+                resp = self._client.cases.get_cases(project_id=project_id)
+                cases = resp.get('cases', [])
+        except StatusCodeError as e:
+            raise ToolException(f"Unable to extract test cases: {e}")
+            # Apply filters
+        if section_id is not None:
+            cases = [case for case in cases if case.get('section_id') == section_id]
+        if title_keyword is not None:
+            cases = [case for case in cases if title_keyword.lower() in case.get('title', '').lower()]
+
+        for case in cases:
+            yield Document(page_content=json.dumps(case), metadata={
+                'project_id': project_id,
+                'title': case.get('title', ''),
+                'suite_id': suite_id or case.get('suite_id', ''),
+                'id': str(case.get('id', '')),
+                IndexerKeywords.UPDATED_ON.value: case.get('updated_on') or -1,
+                'labels': [lbl['title'] for lbl in case.get('labels', [])],
+                'type': case.get('type_id') or -1,
+                'priority': case.get('priority_id') or -1,
+                'milestone': case.get('milestone_id') or -1,
+                'estimate': case.get('estimate') or '',
+                'automation_type': case.get('custom_automation_type') or -1,
+                'section_id': case.get('section_id') or -1,
+                'entity_type': 'test_case',
+            })
+
+    def _process_document(self, document: Document) -> Generator[Document, None, None]:
+        """
+        Process an existing base document to extract relevant metadata for full document preparation.
+        Used for late processing of documents after we ensure that the document has to be indexed to avoid
+        time-consuming operations for documents which might be useless.
+
+        Args:
+            document (Document): The base document to process.
+
+        Returns:
+            Generator[Document, None, None]: A generator yielding processed Document objects with metadata.
+        """
+        try:
+            if not self._include_attachments:
+                # If attachments are not included, return the document as is
+                yield document
+                return
+
+            # get base data from the document required to extract attachments and other metadata
+            base_data = json.loads(document.page_content)
+            case_id = base_data.get("id")
+
+            # get a list of attachments for the case
+            attachments = self._client.attachments.get_attachments_for_case_bulk(case_id=case_id)
+
+            # process each attachment to extract its content
+            for attachment in attachments:
+                if get_file_extension(attachment['filename']) in self._skip_attachment_extensions:
+                    logger.info(f"Skipping attachment {attachment['filename']} with unsupported extension.")
+                    continue
+
+                attachment_id = f"attach_{attachment['id']}"
+                # add attachment id to metadata of parent
+                document.metadata.setdefault(IndexerKeywords.DEPENDENT_DOCS.value, []).append(attachment_id)
+                # TODO: pass it to chunkers
+                yield Document(page_content=self._process_attachment(attachment),
+                                                     metadata={
+                                                         'project_id': base_data.get('project_id', ''),
+                                                         'id': str(attachment_id),
+                                                         IndexerKeywords.PARENT.value: str(case_id),
+                                                         'filename': attachment['filename'],
+                                                         'filetype': attachment['filetype'],
+                                                         'created_on': attachment['created_on'],
+                                                         'entity_type': 'test_case_attachment',
+                                                         'is_image': attachment['is_image'],
+                                                     })
+        except json.JSONDecodeError as e:
+            raise ToolException(f"Failed to decode JSON from document: {e}")
+
+    def _process_attachment(self, attachment: Dict[str, Any]) -> str:
+        """
+        Processes an attachment to extract its content.
+
+        Args:
+            attachment (Dict[str, Any]): The attachment data.
+
+        Returns:
+            str: string description of the attachment.
+        """
+
+        page_content = "This filetype is not supported."
+        if attachment['filetype'] == 'txt' :
+            page_content =  self._client.get(endpoint=f"get_attachment/{attachment['id']}")
+        else:
+            try:
+                attachment_path = self._client.attachments.get_attachment(attachment_id=attachment['id'], path=f"./{attachment['filename']}")
+                page_content = parse_file_content(file_name=attachment['filename'], file_content=attachment_path.read_bytes(), llm=self.llm, is_capture_image=True)
+            except BadRequestError as ai_e:
+                logger.error(f"Unable to parse page's content with type: {attachment['filetype']} due to AI service issues: {ai_e}")
+            except Exception as e:
+                logger.error(f"Unable to parse page's content with type: {attachment['filetype']}: {e}")
+        return page_content
+
+    def _index_tool_params(self):
+        return {
+            'project_id': (str, Field(description="TestRail project ID to index data from")),
+            'suite_id': (Optional[str],
+                         Field(default=None, description="Optional TestRail suite ID to filter test cases")),
+            'section_id': (Optional[int], Field(default=None, description="Optional section ID to filter test cases")),
+            'include_attachments': (Optional[bool],
+                                    Field(description="Whether to include attachment content in indexing",
+                                          default=False)),
+            'skip_attachment_extensions': (Optional[List[str]], Field(
+                description="List of file extensions to skip when processing attachments: i.e. ['.png', '.jpg']",
+                default=[])),
+        }
+
     def _to_markup(self, data: List[Dict], output_format: str) -> str:
         """
         Converts the given data into the specified format: 'json', 'csv', or 'markdown'.
@@ -549,8 +688,9 @@ class TestrailAPIWrapper(BaseToolApiWrapper):
         if output_format == "markdown":
             return df.to_markdown(index=False)
 
+    @extend_with_vector_tools
     def get_available_tools(self):
-        return [
+        tools = [
             {
                 "name": "get_case",
                 "ref": self.get_case,
@@ -586,5 +726,6 @@ class TestrailAPIWrapper(BaseToolApiWrapper):
                 "ref": self.update_case,
                 "description": self.update_case.__doc__,
                 "args_schema": updateCase,
-            },
+            }
         ]
+        return tools
